@@ -12,6 +12,8 @@
 #include <Eris/Exceptions.h>
 #include <Eris/TypeService.h>
 #include <Eris/Operations.h>
+#include <Eris/Response.h>
+#include <Eris/DeleteLater.h>
 
 #include <wfmath/atlasconv.h>
 #include <sigc++/slot.h>
@@ -25,6 +27,7 @@ using Atlas::Objects::Root;
 using Atlas::Objects::Entity::Anonymous;
 using WFMath::TimeStamp;
 using namespace Atlas::Message;
+using Atlas::Objects::smart_dynamic_cast;
 
 namespace Eris
 {
@@ -43,18 +46,28 @@ Avatar::Avatar(Account* pl, const std::string& entId) :
 
     m_view->getEntityFromServer("");
     m_view->getEntity(m_entityId);
-    
-    m_wielded.Changed.connect(sigc::mem_fun(this, &Avatar::onWieldedChanged));
 }
 
 Avatar::~Avatar()
 {
-    m_account->deactivateCharacter(this);
+    m_account->internalDeactivateCharacter(this);
     
     delete m_router;
     delete m_view;
 }
 
+void Avatar::deactivate()
+{
+    Logout l;
+    Anonymous arg;
+    arg->setId(m_entityId);
+    l->setArgs1(arg);
+    l->setSerialno(getNewSerialno());
+    
+    getConnection()->getResponder()->await(l->getSerialno(), this, &Avatar::logoutResponse);
+    getConnection()->send(l);
+}
+
 #pragma mark -
 
 void Avatar::drop(Entity* e, const WFMath::Point<3>& pos, const std::string& loc)
@@ -228,11 +241,9 @@ void Avatar::wield(Entity * entity)
 	}
 	
 	Anonymous arguments;
-	
 	arguments->setId(entity->getId());
 	
 	Wield wield;
-	
 	wield->setFrom(m_entityId);
 	wield->setArgs1(arguments);
 	
@@ -281,6 +292,13 @@ void Avatar::attack(Entity* entity)
     getConnection()->send(attackOp);
 }
 
+void Avatar::useStop()
+{
+    Use use;
+	use->setFrom(m_entityId);
+    getConnection()->send(use);
+}
+
 #pragma mark -
 
 void Avatar::onEntityAppear(Entity* ent)
@@ -341,34 +359,29 @@ void Avatar::updateWorldTime(double seconds)
     m_lastOpTime = seconds;
 }
 
-void Avatar::onWieldedChanged()
+void Avatar::logoutResponse(const RootOperation& op)
 {
-    m_useOps.clear();
-
-    if (!m_wielded) return;
-    if (!m_wielded->hasAttr("operations")) return;
+    if (!op->instanceOf(INFO_NO))
+        warning() << "received an avatar logout response that is not an INFO";
+        
+    const std::vector<Root>& args(op->getArgs());
     
-    const Element& ops = m_wielded->valueOfAttr("operations");
-    if (!ops.isList()) {
-        warning() << "entity " << m_wielded->getId() << 
-            " has operations attr which is not a list";
+    if (args.empty() || (args.front()->getClassNo() != LOGOUT_NO)) {
+        warning() << "argument of avatar logout INFO is not a logout op";
         return;
-    } 
-    
-    const ListType& opsl(ops.asList());
-    m_useOps.reserve(opsl.size());
-    TypeService* ts = getConnection()->getTypeService();
-    
-    for (ListType::const_iterator i=opsl.begin(); i!=opsl.end(); ++i)
-    {
-        if (!i->isString())
-        {
-            warning() << "ignoring malformed operations list item";
-            continue;
-        }
-    
-        m_useOps.push_back(ts->getTypeByName(i->asString()));
     }
+    
+    RootOperation logout = smart_dynamic_cast<RootOperation>(args.front());
+    const std::vector<Root>& args2(logout->getArgs());
+    assert(!args2.empty());
+    
+    std::string charId = args2.front()->getId();
+    debug() << "got logout for character " << charId;
+    assert(charId == m_entityId);
+
+    m_account->AvatarDeactivated.emit(this);
+    deleteLater(this);
 }
 
+
 } // of namespace Eris
