@@ -110,14 +110,12 @@ public:
     void attack(Entity* entity);
     
     /**
-    @brief Get a list of operations supported by the currently wielded entity (tool)
-    If no entity is wielded, or the entity does no provide an operations list,
-    this will return an empty array.
+    @brief Stop the current task, if one is in progress. 
+    This could be either a useOn or attack.
     */
-    const TypeInfoArray& getUseOperationsForWielded() const
-    {
-        return m_useOps;
-    }
+    void useStop();
+    
+    void deactivate();
     
     /**
     Emitted when the character entity of this Avatar is valid (and presumably,
@@ -165,6 +163,8 @@ private:
     void onCharacterWield(const std::string&, const Atlas::Message::Element&);
     void onWieldedChanged();
     
+    void logoutResponse(const Atlas::Objects::Operation::RootOperation&);
+    
     Account* m_account;
     
     std::string m_entityId;
