@@ -2,11 +2,20 @@
 #define ERIS_CALENDAR_H
 
 #include <Eris/Types.h>
-#include <Atlas/Message/Element.h>
 
 #include <sigc++/trackable.h>
 #include <sigc++/connection.h>
 
+#include <map>
+
+namespace Atlas
+{
+namespace Message
+{
+typedef std::map<std::string, Element> MapType;
+}
+}
+
 namespace Eris
 {
 
@@ -29,6 +38,7 @@ public:
     unsigned int seconds() const { return m_seconds; }
     unsigned int minutes() const { return m_minutes; }
     unsigned int hours() const { return m_hours; }
+
 private:
 	friend class Calendar;
     
@@ -50,6 +60,10 @@ public:
     
     DateTime now() const;
     
+    unsigned int secondsPerMinute() const { return m_secondsPerMinute; }
+    unsigned int minutesPerHour() const { return m_minutesPerHour; }
+    unsigned int hoursPerDay() const { return m_hoursPerDay; }
+
 private:
     void topLevelEntityChanged();
     void calendarAttrChanged(const std::string&, const Atlas::Message::Element& value);
