@@ -11,6 +11,7 @@
 #include <Eris/Exceptions.h>
 #include <Eris/Avatar.h>
 #include <Eris/Alarm.h>
+#include <Eris/Task.h>
 
 #include <wfmath/atlasconv.h>
 #include <Atlas/Objects/Entity.h>
@@ -26,6 +27,8 @@ using namespace Atlas::Objects::Operation;
 using Atlas::Objects::Root;
 using Atlas::Objects::Entity::RootEntity;
 using Atlas::Message::Element;
+using Atlas::Message::ListType;
+using Atlas::Message::MapType;
 using Atlas::Objects::smart_static_cast;
 using Atlas::Objects::smart_dynamic_cast;
 
@@ -45,7 +48,8 @@ Entity::Entity(const std::string& id, TypeInfo* ty, View* vw) :
     m_view(vw),
     m_hasBBox(false),
     m_moving(false),
-    m_recentlyCreated(false)
+    m_recentlyCreated(false),
+    m_initialised(true)
 {
     assert(m_id.size() > 0);
     m_orientation.identity();
@@ -55,16 +59,27 @@ Entity::Entity(const std::string& id, TypeInfo* ty, View* vw) :
 }
 
 Entity::~Entity()
-{   
+{
+    assert(m_initialised == false);   
+}
+
+void Entity::shutdown()
+{
     BeingDeleted.emit();
     if (m_moving) m_view->removeFromPrediction(this);
     
-    while (!m_contents.empty()) delete m_contents.back();    
+    while (!m_contents.empty()) {
+      Entity *e = m_contents.back();
+      e->shutdown();
+      delete e;
+    }
     setLocation(NULL);
     
     m_view->getConnection()->unregisterRouterForFrom(m_router, m_id);
     m_view->entityDeleted(this); // remove ourselves from the View's content map
     delete m_router;
+
+    m_initialised = false;
 }
 
 void Entity::init(const RootEntity& ge, bool fromCreateOp)
@@ -125,7 +140,12 @@ WFMath::Quaternion Entity::getViewOrientation() const
 
 WFMath::Point<3> Entity::getPredictedPos() const
 {
-    return (m_moving ? m_predicted.position : m_position);
+    WFMath::Point<3> res = (m_moving ? m_predicted.position : m_position);
+ //   if (!res.isValid())
+ //   {
+ //       debug() << "invalid pos for entity " << m_id << ":" << m_name;
+ //   }
+    return res;
 }
 
 WFMath::Vector<3> Entity::getPredictedVelocity() const
@@ -138,7 +158,6 @@ bool Entity::isMoving() const
     return m_moving;
 }
 
-
 void Entity::updatePredictedState(const WFMath::TimeStamp& t)
 {
     assert(isMoving());
@@ -154,6 +173,35 @@ void Entity::updatePredictedState(const WFMath::TimeStamp& t)
     }
 }
 
+TypeInfoArray Entity::getUseOperations() const
+{
+    AttrMap::const_iterator it = m_attrs.find("operations");
+    if (it == m_attrs.end()) return TypeInfoArray();
+    
+    if (!it->second.isList()) {
+        warning() << "entity " << m_id << " has operations attr which is not a list";
+        return TypeInfoArray();
+    } 
+    
+    const ListType& opsl(it->second.asList());
+    TypeInfoArray useOps;
+    useOps.reserve(opsl.size());
+    TypeService* ts = m_view->getAvatar()->getConnection()->getTypeService();
+    
+    for (ListType::const_iterator i=opsl.begin(); i!=opsl.end(); ++i)
+    {
+        if (!i->isString())
+        {
+            warning() << "ignoring malformed operations list item";
+            continue;
+        }
+    
+        useOps.push_back(ts->getTypeByName(i->asString()));
+    }
+    
+    return useOps;
+}
+
 #pragma mark -
 
 void Entity::sight(const RootEntity &ge)
@@ -319,6 +367,9 @@ bool Entity::nativeAttrChanged(const std::string& attr, const Element& v)
         return true;
     } else if ((attr == "loc") ||(attr == "contains")) {
         throw InvalidOperation("tried to set loc or contains via setProperty");
+    } else if (attr == "tasks") {
+        updateTasks(v);
+        return true;
     }
 
     return false; // not a native property
@@ -369,6 +420,84 @@ void Entity::endUpdate()
     }
 }
 
+static int findTaskByName(const TaskArray& a, const std::string& nm)
+{
+    for (unsigned int t=0; t < a.size(); ++t)
+    {
+        if (a[t]->name() == nm) return t;
+    }
+    
+    return -1;
+}
+
+void Entity::updateTasks(const Element& e)
+{
+    if (!e.isList()) return; // malformed
+    const ListType& taskList(e.asList());
+    
+    /*
+    Explanation:
+    
+    We take a copy of the current pointer-to-task array, clear out the 'real'
+    task array, and then walk the array defined by Atlas. For each pre-existing
+    task still specified in the Atlas data, we find the old task in the copied
+    array, re-add it to the main list, and NULL the entry in the copied list.
+    
+    At the end of iterating the Atlas data, any pointers still valid in the
+    copied list are dead, and can be deleted.
+    */
+    
+    TaskArray previousTasks(m_tasks);
+    m_tasks.clear();
+    
+    for (unsigned int t=0; t<taskList.size(); ++t)
+    {
+        const MapType& tkmap(taskList[t].asMap());
+        MapType::const_iterator it = tkmap.find("name");
+        if (it == tkmap.end())
+        {
+            error() << "task without name";
+            continue;
+        }
+        
+        int index = findTaskByName(previousTasks, it->second.asString());
+        Task *t;
+        
+        if (index < 0)
+        {   // not found, create a new one
+            t = new Task(this, it->second.asString());
+            TaskAdded.emit(t);
+        } else {
+            t = previousTasks[index];
+            previousTasks[index] = NULL;
+        }
+        
+        m_tasks.push_back(t);
+        t->updateFromAtlas(tkmap);
+    } // of Atlas-specified tasks iteration
+    
+    for (unsigned int d=0; d<previousTasks.size(); ++d)
+    {
+        if (!previousTasks[d]) continue;
+        
+        TaskRemoved(previousTasks[d]);
+        delete previousTasks[d];
+    } // of previous-task cleanup iteration
+}
+
+void Entity::removeTask(Task* t)
+{
+    TaskArray::iterator it = std::find(m_tasks.begin(), m_tasks.end(), t);
+    if (it == m_tasks.end())
+    {
+        error() << "unknown task " << t->name() << " on entity " << this;
+        return;
+    }
+    
+    m_tasks.erase(it);
+    TaskRemoved(t);
+}
+
 #pragma mark -
 
 void Entity::setLocationFromAtlas(const std::string& locId)
